"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var windy_1 = require("./windy");
var canvasBound_1 = require("./canvasBound");
var mapBound_1 = require("./mapBound");
var layer_1 = require("./layer");
var ol_1 = require("ol");
// import CanvasLayer from './L.CanvasLayer';
// import VelocityControl from './L.ControlVelocity'
// declare var L: any;
// const L_CanvasLayer = (L.Layer ? L.Layer : L.Class).extend(new CanvasLayer());
// const L_canvasLayer = function() {
//   return new L_CanvasLayer();
// };
// const L_ControlVelocity = (L.Control).extend(new VelocityControl);
// const L_controlVelocity = function() {
//   return new L_ControlVelocity();
// };
var VelocityLayer = /** @class */ (function () {
    function VelocityLayer() {
        this._map = null;
        this._canvas = null;
        this._canvasLayer = null;
        this._windy = null;
        this._context = null;
        this._displayTimeout = 0;
        this._mouseControl = null;
        this.canvasFunction = function (extent, resolution, pixelRatio, size, projection) {
            var canvas = document.createElement('canvas');
            canvas.setAttribute('width', size[0]);
            canvas.setAttribute('height', size[1]);
            this._context = canvas.getContext('2d');
            this._canvas = canvas;
            // Canvas extent is different than map extent, so compute delta between 
            // left-top of map and canvas extent.
            // var mapExtent = map.getView().calculateExtent(map.getSize())
            // var canvasOrigin = map.getPixelFromCoordinate([extent[0], extent[3]]);
            // var mapOrigin = map.getPixelFromCoordinate([mapExtent[0], mapExtent[3]]);
            // var delta = [mapOrigin[0]-canvasOrigin[0], mapOrigin[1]-canvasOrigin[1]]
            return canvas;
        };
        this.options = {
            displayValues: true,
            displayOptions: {
                velocityType: 'Velocity',
                position: 'bottomleft',
                emptyString: 'No velocity data',
                angleConvention: 'bearingCCW',
                speedUnit: 'm/s'
            },
            maxVelocity: 10,
            colorScale: null,
            data: null
        };
    }
    VelocityLayer.prototype.initialize = function (options) {
        for (var i in options) {
            this.options[i] = options[i];
        }
    };
    VelocityLayer.prototype.addToMap = function (map) {
        this._canvasLayer = new ol_1.default.layer.Image({
            source: new ol_1.default.source.ImageCanvas({
                canvasFunction: this.canvasFunction,
                projection: 'EPSG:3857'
            })
        });
        map.addLayer(this._canvasLayer);
        this._map = map;
        this._drawWindy();
    };
    VelocityLayer.prototype.removeFromMap = function (map) {
        this._destroyWind();
        map.removeLayer(this._canvasLayer);
    };
    VelocityLayer.prototype.setData = function (data) {
        this.options.data = data;
        if (this._windy) {
            this._windy.setData(data);
            this._clearAndRestart();
        }
    };
    VelocityLayer.prototype._initWindy = function () {
        var _this = this;
        // windy object, copy options
        var options = Object.assign({ canvas: this._canvas }, this.options);
        this._windy = new windy_1.default(options);
        //TODO : Figure out why the event is called after the layer is removed
        this._map.on('dragstart', function () {
            if (_this._windy)
                _this._windy.stop();
        });
        this._map.on('dragend', function () {
            _this._clearAndRestart();
        });
        this._map.on('zoomstart', function () {
            if (_this._windy)
                _this._windy.stop();
        });
        this._map.on('zoomend', function () {
            _this._clearAndRestart();
        });
        this._map.on('resize', function () {
            _this._clearWind();
        });
        // this._initMouseHandler();
    };
    // _initMouseHandler() {
    //   if (!this._mouseControl && this.options.displayValues) {
    //     var options = this.options.displayOptions || {};
    //     options['leafletVelocity'] = this;
    //     this._mouseControl = L_controlVelocity();
    //     this._mouseControl.setWindy(this._windy);
    //     this._mouseControl.setOptions(this.options.displayOptions);
    //     this._mouseControl.addTo(this._map);
    //   }
    // }
    VelocityLayer.prototype._startWindy = function () {
        var mapSize = this._map.getSize();
        var mapExtent = this._map.getView().calculateExtent(mapSize);
        var extentLL = ol_1.default.proj.transformExtent(mapExtent, this._map.getView().getProjection(), 'EPSG:4326');
        this._windy.start(new layer_1.default(new mapBound_1.default(extentLL[3] / 180 * Math.PI, // maxy (north)
        extentLL[2] / 180 * Math.PI, // maxx (east)
        extentLL[1] / 180 * Math.PI, // miny (south)
        extentLL[0] / 180 * Math.PI // minx (west)
        ), new canvasBound_1.default(0, 0, mapSize[0], mapSize[1])));
    };
    VelocityLayer.prototype._drawWindy = function () {
        if (!this._windy) {
            this._initWindy();
            return;
        }
        if (!this.options.data) {
            return;
        }
        var self = this;
        if (this._displayTimeout)
            clearTimeout(self._displayTimeout);
        this._displayTimeout = setTimeout(function () {
            self._startWindy();
        }, 150); // showing velocity is delayed
    };
    VelocityLayer.prototype._clearAndRestart = function () {
        if (this._context)
            this._context.clearRect(0, 0, 3000, 3000);
        if (this._windy)
            this._startWindy();
    };
    VelocityLayer.prototype._clearWind = function () {
        if (this._windy)
            this._windy.stop();
        if (this._context)
            this._context.clearRect(0, 0, 3000, 3000);
    };
    VelocityLayer.prototype._destroyWind = function () {
        if (this._displayTimeout)
            clearTimeout(this._displayTimeout);
        if (this._windy)
            this._windy.stop();
        if (this._context)
            this._context.clearRect(0, 0, 3000, 3000);
        // if (this._mouseControl)
        //   this._map.removeControl(this._mouseControl);
        // this._mouseControl = null;
        this._windy = null;
        // this._map.removeLayer(this._canvasLayer);
    };
    return VelocityLayer;
}());
exports.default = VelocityLayer;
